/*
 * This file is part of aion-lightning <aion-lightning.com>.
 *
 *  aion-lightning is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  aion-lightning is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with aion-lightning.  If not, see <http://www.gnu.org/licenses/>.
 */
package mysql5;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.aionemu.commons.database.DatabaseFactory;
import com.aionemu.gameserver.dao.EventInfoDAO;
import com.aionemu.gameserver.dao.MySQL5DAOUtils;
import com.aionemu.gameserver.model.gameobjects.PersistentState;
import com.aionemu.gameserver.model.gameobjects.player.Player;
import com.aionemu.gameserver.services.custom.EventType;
import com.aionemu.gameserver.services.custom.PlayerEvent;
import com.aionemu.gameserver.services.custom.PlayerEventInfo;

/**
 * @author zxl001
 */
public class MySQL5EventInfoDAO extends EventInfoDAO {
	private static final Logger log = LoggerFactory.getLogger(MySQL5EventInfoDAO.class);
	
	public static final String LOAD_QUERY = "SELECT * FROM `event_info` WHERE `object_id`=?";
	public static final String UPDATA_QUERY = "UPDATE event_info SET count=?, use_count=?";
	
	@Override
	public boolean supports(String databaseName, int majorVersion, int minorVersion) {
		return MySQL5DAOUtils.supports(databaseName, majorVersion, minorVersion);
	}

	@Override
	public void loadEventInfo(Player player) {
		
		final Map<EventType, PlayerEvent> infos = new HashMap<EventType, PlayerEvent>();
		final PlayerEventInfo info = new PlayerEventInfo(player);
		
		Connection con = null;
		try {
			con = DatabaseFactory.getConnection();
			PreparedStatement stmt = con.prepareStatement(LOAD_QUERY);
			stmt.setInt(1, player.getObjectId());
			ResultSet rset = stmt.executeQuery();
			while(rset.next()) {
				EventType type = EventType.valueOf(rset.getString("event_type"));
				int count = rset.getInt("count");
				int usecount = rset.getInt("use_count");
				infos.put(type, new PlayerEvent(type, count, usecount));
			}
			info.setEvents(infos);
			player.setPlayerEventInfo(info);
			rset.close();
			stmt.close();
		}
		catch (Exception e) {
			log.error("", e);
		}
		finally {
			DatabaseFactory.close(con);
		}
	}

	@Override
	public void delete() {
		Connection con = null;
		try {
			con = DatabaseFactory.getConnection();
			PreparedStatement stmt = con.prepareStatement(UPDATA_QUERY);
			stmt.setInt(1, 0);
			stmt.setInt(2, 0);
			stmt.execute();
			stmt.close();
		}
		catch (Exception e) {
			log.error("", e);
		}
		finally{
			DatabaseFactory.close(con);
		}
	}

	@Override
	public void store(Player player) {
		Collection<PlayerEvent> temps = player.getEventInfo().getEvents();
		Connection con = null;
		try {
			con = DatabaseFactory.getConnection();
			con.setAutoCommit(false);
			for(PlayerEvent event : temps) {
				if (event.getPersistentState() == PersistentState.NEW) {
					insertEvent(con, player.getObjectId(), event);
				}
				if (event.getPersistentState() == PersistentState.UPDATE_REQUIRED) {
					updataEvent(con, player.getObjectId(), event);
				}
			}
			for(PlayerEvent event : temps) {
				event.setPersistentState(PersistentState.UPDATED);
			}
		}
		catch (Exception e) {
			// TODO: handle exception
		}
		finally {
			DatabaseFactory.close(con);
		}
	}
	
	private void insertEvent(Connection con, int object_id, PlayerEvent event) {
		PreparedStatement stmt = null;
		try {
			stmt = con.prepareStatement("INSERT INTO event_info(object_id, event_type, count, use_count)" + "VALUES (?, ?, ?, ?)");
			stmt.setInt(1, object_id);
			stmt.setString(2, event.getEventType().toString());
			stmt.setInt(3, event.getCount());
			stmt.setInt(4, event.getUseCount());
			stmt.execute();
			stmt.close();
		}
		catch (Exception e) {
			log.error("", e);
		}
	}
	
	private void updataEvent(Connection con, int object_id, PlayerEvent event) {
		PreparedStatement stmt = null;
		try {
			stmt = con.prepareStatement("UPDATE event_info SET count=?, use_count=? WHERE object_id=? AND event_type=?");
			stmt.setInt(1, event.getCount());
			stmt.setInt(2, event.getUseCount());
			stmt.setInt(3, object_id);
			stmt.setString(4, event.getEventType().toString());
			stmt.execute();
			stmt.close();
		}
		catch (Exception e) {
			log.error("", e);
		}
	}
}
