/*
 * This file is part of aion-lightning <aion-lightning.com>.
 *
 *  aion-lightning is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  aion-lightning is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with aion-lightning.  If not, see <http://www.gnu.org/licenses/>.
 */
package mysql5;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.aionemu.commons.database.DatabaseFactory;
import com.aionemu.gameserver.dao.CPointUseDAO;
import com.aionemu.gameserver.dao.MySQL5DAOUtils;
import com.aionemu.gameserver.model.gameobjects.PersistentState;
import com.aionemu.gameserver.model.gameobjects.player.CPUsePoint;
import com.aionemu.gameserver.model.gameobjects.player.CPointUseList;
import com.aionemu.gameserver.model.gameobjects.player.Player;

/**
 * @author zxl001
 */
public class MySQL5UseCPointDAO extends CPointUseDAO {

	private static final Logger log = LoggerFactory.getLogger(MySQL5UseCPointDAO.class);

	public static final String SELECT_QUERY = "SELECT * FROM `player_cpoint` WHERE `object_id`=?";
	public static final String DELETE_QUERY = "DELETE FROM `player_cpoint` WHERE `object_id`=? AND `stat_id`=?";
	public static final String UPDATA_QUERY = "UPDATE player_cpoint SET point=? WHERE `object_id`=? AND `stat_id`=?";
	public static final String INSERT_QUERY = "INSERT INTO `player_cpoint` (`object_id`, `stat_id`, `point`) VALUES(?, ?, ?)";

	@Override
	public boolean supports(String databaseName, int majorVersion, int minorVersion) {
		return MySQL5DAOUtils.supports(databaseName, majorVersion, minorVersion);
	}

	@Override
	public void loadUseCPoint(Player player) {
		CPointUseList list = new CPointUseList();
		Connection con = null;
		try {
			con = DatabaseFactory.getConnection();
			PreparedStatement stmt = con.prepareStatement(SELECT_QUERY);
			stmt.setInt(1, player.getObjectId());
			ResultSet rset = stmt.executeQuery();
			while (rset.next()) {
				int statid = rset.getInt("stat_id");
				int point = rset.getInt("point");
				CPUsePoint cpUsePoint = new CPUsePoint(statid, point);
				list.AddCPUsePoint(cpUsePoint);
			}
			player.setCpUseList(list);
			rset.close();
			stmt.close();
		}
		catch (Exception e) {
			log.error("Could not restore cp use point for player: " + player.getName() + " from DB: " + e.getMessage(), e);
		}
		finally {
			DatabaseFactory.close(con);
		}
	}

	@Override
	public void updataUseCPoint(int object_id, CPUsePoint cp) {

		Connection con = null;
		try {
			con = DatabaseFactory.getConnection();
			con.setAutoCommit(false);
			boolean result = false;
			switch (cp.getPersistentState()) {
				case NEW:
					result = insertCp(con, object_id, cp);
					break;
				case DELETED:
					result = deleteCp(con, object_id, cp);
					break;
				case UPDATE_REQUIRED:
					result = updataCp(con, object_id, cp);
					break;
			}
			if (result)
				cp.setPersistentState(PersistentState.UPDATED);
		}
		catch (Exception e) {
			log.error("can't updata cp for player: " + object_id + " from DB: " + e.getMessage(), e);
		}
		finally {
			DatabaseFactory.close(con);
		}
	}

	private boolean insertCp(Connection con, int object_id, CPUsePoint cp) {
		PreparedStatement stmt = null;
		try {
			stmt = con.prepareStatement(INSERT_QUERY);
			stmt.setInt(1, object_id);
			stmt.setInt(2, cp.getId());
			stmt.setInt(3, cp.getPoint());
			stmt.executeUpdate();
			con.commit();
			return true;
		}
		catch (Exception e) {
			log.error("can't add cp for player: " + object_id + " from DB: " + e.getMessage(), e);
			return false;
		}
		finally {
			DatabaseFactory.close(stmt);
		}
	}

	private boolean deleteCp(Connection con, int object_id, CPUsePoint cp) {
		PreparedStatement stmt = null;
		try {
			stmt = con.prepareStatement(DELETE_QUERY);

			stmt.setInt(1, object_id);
			stmt.setInt(2, cp.getId());
			stmt.executeUpdate();
			con.commit();
			return true;
		}
		catch (Exception e) {
			log.error("can't delete cp for player: " + object_id + " from DB: " + e.getMessage(), e);
			return false;
		}
		finally {
			DatabaseFactory.close(stmt);
		}
	}

	private boolean updataCp(Connection con, int object_id, CPUsePoint cp) {
		PreparedStatement stmt = null;
		try {
			stmt = con.prepareStatement(UPDATA_QUERY);
			stmt.setInt(1, cp.getPoint());
			stmt.setInt(2, object_id);
			stmt.setInt(3, cp.getId());
			stmt.executeUpdate();
			con.commit();
			return true;
		}
		catch (Exception e) {
			log.error("can't updata cp for player: " + object_id + " from DB: " + e.getMessage(), e);
			return false;
		}
		finally {
			DatabaseFactory.close(stmt);
		}
	}
}
